// %W%
#ifndef _DN2DOC_
#define _DN2DOC_

// Definition of the internal storage of a document
#include <vector>
#include <string>
#include <fstream>
#include "SGMLApplication.h"

class AttributeClass {
 public:
  AttributeClass();  // default constructor
  AttributeClass(const SGMLApplication::CharString *, const SGMLApplication::CharString *); // ctor w/ 2params
  AttributeClass(const char *, const char *); // ctor w/ 2 char * params
  ~AttributeClass(); // destructor
  //AttributeClass(const AttributeClass &); // copy constructor

  string get_attributename() const;
  string get_attributevalue() const;
  void set_attributename(const SGMLApplication::CharString *);
  void set_attributevalue(const SGMLApplication::CharString *);
  void set_attributename(const char *);
  void set_attributevalue(const char *);

 private:
  string AttributeName;
  string AttributeValue;
};

class DataClass {
 public:
  DataClass(); // default constructor
  DataClass(const SGMLApplication::CharString *);  // constructor with CharString data
  DataClass(const char *);  // constructor with char * data
  ~DataClass();  // destructor
  DataClass(const DataClass &);  // copy constructor

  bool operator==(const DataClass &);

  string get_data() const;
  void set_data(const SGMLApplication::CharString *);
  void set_data(const char *);

 private:
  string data;
};

class DocumentData {

 public:
  enum NodeEnum {EMPTY, APP_SE, SE_APP, SE_APP_HELP, SE_APP_EXPL, SE_APP_DOCFREQ, SE_APP_COLLFREQ,
		   SE_APP_INTERNAL_ID, SE_APP_MATCHES, CONTEXT,
		   FULL_TERM, TEXT, INDEPENDENT, AND, OR, AND_NOT,
		   HEAD_RELATION, OTHER_OPER, INFO_NEED, DOCID, DOC_COLLECTION, B_AND, B_OR, B_AND_NOT,
		   RESTRICT_SET, FEEDBACK_INFO, DOCID_REL, TEXT_REL, COMMENT, ANN_ATTR, ANN_TYPE,
		   ATTR_NAME, ATTR_TYPE, ATTR_VALUE, MERGE_INFO, DN2};

  DocumentData();
  ~DocumentData();


  void start_node(const NodeEnum type_of_node);
  void start_node(const string type_of_node);
  void start_node(const char *type_of_node);
  void start_node(const SGMLApplication::CharString type_of_node);
  void add_data(const SGMLApplication::CharString *data);
  void add_data(const char *);
  void add_attribute(const SGMLApplication::CharString *attribname,
		     const SGMLApplication::CharString *attribvalue);
  void add_attribute(const char *, const char *);
  void end_node();

  void error_cleanup();

  string return_dn2();
  string return_tcldn2();

  int parse_error;

 private:

  struct NodeStruct {
    NodeEnum nodetype;
    vector<AttributeClass> attributevector;
    vector<DataClass> datavector;
    NodeStruct *parent;
    vector<NodeStruct *> children;
  };

  static const char * const NodeNameArray[];

  NodeStruct head;
  NodeStruct *current;
  string get_nodedata(const NodeStruct *);
  string get_nodeattributes(const NodeStruct *);
  string get_nodetype(const NodeStruct *);
  string get_nodes(const NodeStruct *);

  string get_tclnodes(const NodeStruct *);
  string get_tclnodeattribs(const NodeStruct *);

  string convert_to_tclstring(const string&);
};

#endif
